# -*- coding: utf-8 -*-
from django.apps import AppConfig, apps

from bulk_signals import signals


class BulkSignalsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "bulk_signals"

    def ready(self):
        # monkey patch bulk method handling into the queryset
        from django.db.models.query import QuerySet

        base_bulk_create = QuerySet.bulk_create

        def bulk_create(queryset, objects, **kwargs):
            # get model label from queryset
            model = apps.get_model(queryset.model._meta.label)

            signals.pre_bulk_create.send(sender=model, objects=objects, **kwargs)
            created_objects = base_bulk_create(queryset, objects, **kwargs)
            signals.post_bulk_create.send(sender=model, objects=objects, **kwargs)

            return created_objects

        QuerySet.bulk_create = bulk_create

        base_bulk_update = QuerySet.bulk_update

        def bulk_update(queryset, objects, fields, **kwargs):
            queryset._hints["is_bulk_update"] = True
            model = apps.get_model(queryset.model._meta.label)

            signals.pre_bulk_update.send(
                sender=model, objects=objects, fields=fields, **kwargs
            )
            return_value = base_bulk_update(queryset, objects, fields, **kwargs)
            signals.post_bulk_update.send(
                sender=model, objects=objects, fields=fields, **kwargs
            )

            return return_value

        QuerySet.bulk_update = bulk_update

        base_update = QuerySet.update

        def update(queryset, **kwargs):
            model = apps.get_model(queryset.model._meta.label)

            # if this update is part of a bulk_update action skip this part
            if queryset._hints.get("is_bulk_update", False):
                return base_update(queryset, **kwargs)

            signals.pre_query_update.send(
                sender=model, queryset=queryset, update_kwargs=kwargs
            )
            return_val = base_update(queryset, **kwargs)
            signals.post_query_update.send(
                sender=model, queryset=queryset, update_kwargs=kwargs
            )

            return return_val

        QuerySet.update = update



# -*- coding: utf-8 -*-
from django.db import models
from django.dispatch import receiver

from bulk_signals import signals


def create_stub(*args, **kwargs):
    pass


@receiver(signals.post_bulk_create)
def call_create_stub(*args, **kwargs):
    create_stub(*args, **kwargs)


def update_stub(*args, **kwargs):
    pass


@receiver(signals.post_bulk_update)
def call_update_stub(*args, **kwargs):
    update_stub(*args, **kwargs)


def query_update_stub(*args, **kwargs):
    pass


@receiver(signals.pre_query_update)
@receiver(signals.post_query_update)
def call_query_update_stub(*args, **kwargs):
    query_update_stub(*args, **kwargs)


class BulkTestModel(models.Model):
    num = models.IntegerField(default=0)



# -*- coding: utf-8 -*-
# Generated by Django 3.2.8 on 2021-10-26 13:45

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="BulkTestModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("num", models.IntegerField(default=0)),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
import pytest
from django.db.models import Sum
from django.dispatch import receiver

from bulk_signals import signals

from .models import BulkTestModel

pytestmark = pytest.mark.django_db


@pytest.fixture()
def objects():
    BulkTestModel.objects.bulk_create([BulkTestModel() for _ in range(10)])
    return BulkTestModel.objects.all()


def test_fixture(objects):
    assert BulkTestModel.objects.count() == 10
    assert BulkTestModel.objects.aggregate(sum=Sum("num"))["sum"] == 0


def test_bulk_create(mocker):
    create_stub = mocker.patch("bulk_signals.tests.models.create_stub")
    bulk_update_stub = mocker.patch("bulk_signals.tests.models.update_stub")
    update_stub = mocker.patch("bulk_signals.tests.models.query_update_stub")

    BulkTestModel.objects.bulk_create([BulkTestModel() for _ in range(10)])

    assert bulk_update_stub.call_count == 0
    assert update_stub.call_count == 0
    assert create_stub.call_count == 1


def test_bulk_update(mocker, objects):
    create_stub = mocker.patch("bulk_signals.tests.models.create_stub")
    bulk_update_stub = mocker.patch("bulk_signals.tests.models.update_stub")
    update_stub = mocker.patch("bulk_signals.tests.models.query_update_stub")

    for o in objects:
        o.num = 1

    BulkTestModel.objects.bulk_update(objects, ["num"])

    assert BulkTestModel.objects.aggregate(sum=Sum("num"))["sum"] == 10
    assert bulk_update_stub.call_count == 1
    assert update_stub.call_count == 0
    assert create_stub.call_count == 0


def test_update(mocker, objects):
    create_stub = mocker.patch("bulk_signals.tests.models.create_stub")
    bulk_update_stub = mocker.patch("bulk_signals.tests.models.update_stub")
    update_stub = mocker.patch("bulk_signals.tests.models.query_update_stub")

    BulkTestModel.objects.update(num=1)

    assert BulkTestModel.objects.aggregate(sum=Sum("num"))["sum"] == 10
    assert bulk_update_stub.call_count == 0
    assert update_stub.call_count == 2
    assert create_stub.call_count == 0


def test_pre_bulk_create():
    @receiver(signal=signals.pre_bulk_create)
    def double(sender, objects, *args, **kwargs):
        for o in objects:
            o.num = o.num * 2

    BulkTestModel.objects.bulk_create([BulkTestModel(num=1)])

    assert BulkTestModel.objects.get().num == 2


def test_pre_bulk_update(mocker):
    update_stub = mocker.patch("bulk_signals.tests.models.query_update_stub")

    obj = BulkTestModel.objects.create(num=1)

    obj.num = 2

    @receiver(signal=signals.pre_bulk_update)
    def triple(sender, objects, *args, **kwargs):
        for o in objects:
            o.num = o.num * 3

    BulkTestModel.objects.bulk_update([obj], fields=["num"])

    assert BulkTestModel.objects.get().num == 6
    assert update_stub.call_count == 0


def test_pre_query_update():
    BulkTestModel.objects.create(num=1)

    @receiver(signal=signals.pre_query_update)
    def half(sender, queryset, update_kwargs, **kwargs):
        update_kwargs["num"] /= 2

    BulkTestModel.objects.update(num=8)

    assert BulkTestModel.objects.get().num == 4



# -*- coding: utf-8 -*-
from django.dispatch import Signal

pre_bulk_create = Signal()
post_bulk_create = Signal()
pre_bulk_update = Signal()
post_bulk_update = Signal()
pre_query_update = Signal()
post_query_update = Signal()



# -*- coding: utf-8 -*-
import os
from distutils.core import setup

from setuptools import find_packages

from bulk_signals import __version__

# User-friendly description from README.md
current_directory = os.path.dirname(os.path.abspath(__file__))
try:
    with open(os.path.join(current_directory, "README.md"), encoding="utf-8") as f:
        long_description = f.read()
except Exception:
    long_description = ""


setup(
    # Name of the package
    name="django-bulk-signals",
    # Packages to include into the distribution
    packages=find_packages(".", exclude=["tests"]),
    # Start with a small number and increase it with
    # every change you make https://semver.org
    version=__version__,
    # Chose a license from here: https: //
    # help.github.com / articles / licensing - a -
    # repository. For example: MIT
    license="MIT",
    # Short description of your library
    description="A product aggregation function to a postgres database and makes it available with django",
    # Long description of your library
    long_description=long_description,
    long_description_content_type="text/markdown",
    # Your name
    author="Axel Wegener",
    # Your email
    author_email="pypi@sparse-space.de",
    # Either the link to your github or to your website
    url="https://github.com/awmath/django-bulk-signals",
    # Link from which the project can be downloaded
    download_url="https://github.com/awmath/django-bulk-signals",
    # List of keywords
    keywords=["django"],
    # List of packages to install with this one
    install_requires=["django>3"],
    python_requires=">=3",
    # https://pypi.org/classifiers/
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)



# -*- coding: utf-8 -*-
from django.apps import AppConfig


class BuildingTestConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "bulk_signals.tests"
    label = "bulk_signals_test"



# -*- coding: utf-8 -*-
import environ

env = environ.Env()

DEBUG = True
SECRET_KEY = "b76f64b63954cb6b9daff50508a697b4"
ROOT_URLCONF = __name__

INSTALLED_APPS = ["bulk_signals", "bulk_signals.tests"]

urlpatterns = []

DATABASES = {
    "default": env.db_url("SQLITE_URL", default="sqlite:////tmp/my-tmp-sqlite.db"),
}
